Package: babebi
Type: Package
Title: Bayesian Estimation and Validation for Small-N Designs with
        Rater Bias
Version: 0.1.0
Authors@R: c(
    person(
      given = "Irene",
      family = "Gianeselli",
      email = "irene.gianeselli@unibz.it",
      role = c("aut", "cre"),
      comment = c(
        ORCID = "0000-0002-8689-3035",
        affiliation = "Free University of Bozen-Bolzano"
      )
    ),
    person(
      given = "Andrea",
      family = "Bosco",
      role = "aut",
      comment = c(
        ORCID = "0000-0002-9458-3993",
        affiliation = "University of Bari Aldo Moro"
      )
    ),
    person(
      given = "Demis",
      family = "Basso",
      role = "aut",
      comment = c(
        ORCID = "0000-0002-4595-3513",
        affiliation = "Free University of Bozen-Bolzano"
      )
    )
  )
Description: Approximate Bayesian inference and Monte Carlo validation for
    small-N repeated-measures designs with two time points and two raters.
    The package is intended for applications in which sample size is limited
    and the observed outcome may be affected by rater-specific bias.
    User-supplied data are standardised into a common long-format structure.
    Pre-post effects are analysed using difference scores in a linear model
    with a rater indicator as covariate. Posterior summaries for the
    regression coefficients are obtained from a large-sample normal
    approximation centred at the least-squares estimate with plug-in
    covariance under a flat improper prior. Evidence for a non-zero
    pre-post effect, adjusted for rater differences, is summarised using a
    BIC-based approximation to the Bayes factor for comparison between
    models with and without the pre-post effect. Monte Carlo validation uses
    design quantities estimated from the observed data, including sample
    size, mean pre-post change, and second-rater additive discrepancy, and
    summarises inferential performance in terms of bias, root mean squared
    error, credible interval coverage, posterior tail probabilities, and
    mean Bayes factor values. For background on the BIC approximation and
    Bayes factors, see Schwarz (1978)
    <doi:10.1214/aos/1176344136> and Kass and Raftery (1995)
    <doi:10.1080/01621459.1995.10476572>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: stats, graphics
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-22 12:33:34 UTC; irene
Author: Irene Gianeselli [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8689-3035>, affiliation: Free
    University of Bozen-Bolzano),
  Andrea Bosco [aut] (ORCID: <https://orcid.org/0000-0002-9458-3993>,
    affiliation: University of Bari Aldo Moro),
  Demis Basso [aut] (ORCID: <https://orcid.org/0000-0002-4595-3513>,
    affiliation: Free University of Bozen-Bolzano)
Maintainer: Irene Gianeselli <irene.gianeselli@unibz.it>
Repository: CRAN
Date/Publication: 2026-04-23 20:10:09 UTC
Built: R 4.6.0; ; 2026-04-26 11:59:02 UTC; unix

Package: babebi
Type: Package
Title: Bayesian Estimation and Validation for Small-N Designs with
        Rater Bias
Version: 0.1.0
Authors@R: c(
    person(
      given = "Irene",
      family = "Gianeselli",
      email = "irene.gianeselli@unibz.it",
      role = c("aut", "cre"),
      comment = c(
        ORCID = "0000-0002-8689-3035",
        affiliation = "Free University of Bozen-Bolzano"
      )
    ),
    person(
      given = "Andrea",
      family = "Bosco",
      role = "aut",
      comment = c(
        ORCID = "0000-0002-9458-3993",
        affiliation = "University of Bari Aldo Moro"
      )
    ),
    person(
      given = "Demis",
      family = "Basso",
      role = "aut",
      comment = c(
        ORCID = "0000-0002-4595-3513",
        affiliation = "Free University of Bozen-Bolzano"
      )
    )
  )
Description: Approximate Bayesian inference and Monte Carlo validation for
    small-N repeated-measures designs with two time points and two raters.
    The package is intended for applications in which sample size is limited
    and the observed outcome may be affected by rater-specific bias.
    User-supplied data are standardised into a common long-format structure.
    Pre-post effects are analysed using difference scores in a linear model
    with a rater indicator as covariate. Posterior summaries for the
    regression coefficients are obtained from a large-sample normal
    approximation centred at the least-squares estimate with plug-in
    covariance under a flat improper prior. Evidence for a non-zero
    pre-post effect, adjusted for rater differences, is summarised using a
    BIC-based approximation to the Bayes factor for comparison between
    models with and without the pre-post effect. Monte Carlo validation uses
    design quantities estimated from the observed data, including sample
    size, mean pre-post change, and second-rater additive discrepancy, and
    summarises inferential performance in terms of bias, root mean squared
    error, credible interval coverage, posterior tail probabilities, and
    mean Bayes factor values. For background on the BIC approximation and
    Bayes factors, see Schwarz (1978)
    <doi:10.1214/aos/1176344136> and Kass and Raftery (1995)
    <doi:10.1080/01621459.1995.10476572>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: stats, graphics
Suggests: testthat (>= 3.0.0), knitr, rmarkdown
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-22 12:33:34 UTC; irene
Author: Irene Gianeselli [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8689-3035>, affiliation: Free
    University of Bozen-Bolzano),
  Andrea Bosco [aut] (ORCID: <https://orcid.org/0000-0002-9458-3993>,
    affiliation: University of Bari Aldo Moro),
  Demis Basso [aut] (ORCID: <https://orcid.org/0000-0002-4595-3513>,
    affiliation: Free University of Bozen-Bolzano)
Maintainer: Irene Gianeselli <irene.gianeselli@unibz.it>
Repository: CRAN
Date/Publication: 2026-04-23 20:10:09 UTC
Built: R 4.6.0; ; 2026-04-25 16:43:50 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,babebi_fit)
S3method(print,babebi_fit)
S3method(print,babebi_validation)
S3method(print,summary.babebi_fit)
S3method(summary,babebi_fit)
export(babebi)
export(fit_model)
export(montecarlo_from_data)

# Basic babebi workflow
# ---------------------
# This script illustrates the recommended user workflow:
# 1. organise data in long format
# 2. run the full babebi workflow
# 3. inspect model fit results
# 4. inspect Monte Carlo validation results

# Example data in long format:
# one row = one subject x time x rater observation
dat <- data.frame(
  id = rep(1:4, each = 4),
  time = rep(c("pre", "pre", "post", "post"), times = 4),
  rater = rep(c("r1", "r2"), times = 8),
  y = c(
    3.0, 3.2, 3.8, 4.0,
    2.9, 3.1, 3.5, 3.7,
    3.4, 3.6, 4.0, 4.1,
    3.1, 3.3, 3.9, 4.0
  )
)

# Inspect the input structure
dat

# Run the full babebi workflow:
# - fit the analytical Bayesian model
# - run Monte Carlo validation calibrated on the observed design
res <- babebi(
  data = dat,
  id = id,
  time = time,
  rater = rater,
  outcome = y,
  time_order = c("pre", "post"),
  rater_order = c("r1", "r2"),
  validate = TRUE,
  R = 100,
  n_draws = 500,
  seed = 123
)

# The returned object is a list with:
# - res$fit
# - res$validation
# - res$call
names(res)

# ------------------------------------------------------------------
# 1. Inspect model fit results
# ------------------------------------------------------------------
print(res$fit)
summary(res$fit)
plot(res$fit)

# Access individual fit components directly if needed
res$fit$beta_hat
res$fit$gamma_hat
res$fit$bf10

# ------------------------------------------------------------------
# 2. Inspect Monte Carlo validation results
# ------------------------------------------------------------------
print(res$validation)

# Access validation metrics directly if needed
res$validation$bias
res$validation$rmse
res$validation$coverage
res$validation$ppos_mean
res$validation$bf_mean

# ------------------------------------------------------------------
# 3. Optional: run only the analytical model, without validation
# ------------------------------------------------------------------
res_fit_only <- babebi(
  data = dat,
  id = id,
  time = time,
  rater = rater,
  outcome = y,
  time_order = c("pre", "post"),
  rater_order = c("r1", "r2"),
  validate = FALSE,
  n_draws = 500,
  seed = 123
)

print(res_fit_only$fit)
res_fit_only$validation

#' Standardise user data for babebi
#'
#' Internal helper used to convert user-supplied data into the format
#' required by the estimation and Monte Carlo engine.
#'
#' The function assumes a fully observed 2 (time) × 2 (rater) repeated-measures
#' design. Each subject must contribute exactly one observation for each
#' combination of time point and rater.
#'
#' If `time_order` or `rater_order` are not provided, the ordering of factor
#' levels follows the order of appearance in the data. Users are encouraged
#' to specify these arguments explicitly when the ordering carries substantive
#' meaning (e.g., pre vs post).
#'
#' @param data A data frame in long format.
#' @param id Unquoted column name identifying subjects or a character string.
#' @param time Unquoted column name identifying time points or a character string.
#' @param rater Unquoted column name identifying raters or a character string.
#' @param outcome Unquoted column name containing the observed score or a character string.
#' @param time_order Optional character vector of length 2 specifying the
#'   order of the time factor (e.g., `c("pre", "post")`).
#' @param rater_order Optional character vector of length 2 specifying the
#'   order of the rater factor (reference rater first).
#'
#' @return A data frame with four standardised columns: `id`, `time`,
#'   `rater`, and `y`.
#'
#' @keywords internal
prepare_data <- function(data,
                         id,
                         time,
                         rater,
                         outcome,
                         time_order = NULL,
                         rater_order = NULL) {

  id_name <- resolve_colname(substitute(id), parent.frame())
  time_name <- resolve_colname(substitute(time), parent.frame())
  rater_name <- resolve_colname(substitute(rater), parent.frame())
  outcome_name <- resolve_colname(substitute(outcome), parent.frame())

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }

  required_cols <- c(id_name, time_name, rater_name, outcome_name)
  missing_cols <- setdiff(required_cols, names(data))

  if (length(missing_cols) > 0) {
    stop(
      "Missing required columns: ",
      paste(missing_cols, collapse = ", "),
      call. = FALSE
    )
  }

  dat <- data[, required_cols, drop = FALSE]
  names(dat) <- c("id", "time", "rater", "y")

  if (anyNA(dat)) {
    stop(
      "Missing values in `id`, `time`, `rater`, or `outcome` are not supported.",
      call. = FALSE
    )
  }

  if (!is.numeric(dat$y)) {
    stop("`outcome` must be numeric.", call. = FALSE)
  }

  dat$id <- as.factor(dat$id)
  dat$time <- as.character(dat$time)
  dat$rater <- as.character(dat$rater)

  observed_time <- unique(dat$time)
  observed_rater <- unique(dat$rater)

  if (length(observed_time) != 2) {
    stop(
      "`time` must contain exactly two distinct values. Found: ",
      paste(observed_time, collapse = ", "),
      call. = FALSE
    )
  }

  if (length(observed_rater) != 2) {
    stop(
      "`rater` must contain exactly two distinct values. Found: ",
      paste(observed_rater, collapse = ", "),
      call. = FALSE
    )
  }

  if (is.null(time_order)) {
    time_order <- observed_time
  }

  if (is.null(rater_order)) {
    rater_order <- observed_rater
  }

  if (length(time_order) != 2) {
    stop("`time_order` must have length 2.", call. = FALSE)
  }

  if (length(rater_order) != 2) {
    stop("`rater_order` must have length 2.", call. = FALSE)
  }

  if (!setequal(time_order, observed_time)) {
    stop(
      "`time_order` must contain exactly the same values as the observed `time` levels. Observed: ",
      paste(observed_time, collapse = ", "),
      call. = FALSE
    )
  }

  if (!setequal(rater_order, observed_rater)) {
    stop(
      "`rater_order` must contain exactly the same values as the observed `rater` levels. Observed: ",
      paste(observed_rater, collapse = ", "),
      call. = FALSE
    )
  }

  dat$time <- factor(dat$time, levels = time_order)
  dat$rater <- factor(dat$rater, levels = rater_order)

  counts <- stats::xtabs(~ id + time + rater, data = dat)

  if (any(counts != 1L)) {
    stop(
      "Each subject must have exactly one observation for each combination of time and rater.",
      call. = FALSE
    )
  }

  dat
}

library(testthat)
library(babebi)

test_that("prepare_data standardises a valid dataset correctly", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  out <- babebi:::prepare_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2")
  )

  expect_s3_class(out, "data.frame")
  expect_identical(names(out), c("id", "time", "rater", "y"))
  expect_true(is.factor(out$id))
  expect_true(is.factor(out$time))
  expect_true(is.factor(out$rater))
  expect_identical(levels(out$time), c("pre", "post"))
  expect_identical(levels(out$rater), c("r1", "r2"))
  expect_true(is.numeric(out$y))
  expect_equal(nrow(out), 16)
})

test_that("prepare_data fails when required columns are missing", {
  dat <- data.frame(
    id = rep(1:2, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 2),
    y = c(1, 2, 3, 4, 5, 6, 7, 8)
  )

  expect_error(
    babebi:::prepare_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y
    ),
    "Missing required columns"
  )
})

test_that("prepare_data fails when outcome is not numeric", {
  dat <- data.frame(
    id = rep(1:2, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 2),
    rater = rep(c("r1", "r2"), times = 4),
    y = as.character(1:8)
  )

  expect_error(
    babebi:::prepare_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y
    ),
    "`outcome` must be numeric"
  )
})

test_that("prepare_data fails when time has more than two distinct values", {
  dat <- data.frame(
    id = c(1, 1, 1, 2, 2, 2),
    time = c("pre", "mid", "post", "pre", "mid", "post"),
    rater = c("r1", "r1", "r1", "r2", "r2", "r2"),
    y = c(1, 2, 3, 4, 5, 6)
  )

  expect_error(
    babebi:::prepare_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y
    ),
    "`time` must contain exactly two distinct values"
  )
})

test_that("prepare_data fails when the design is not fully observed", {
  dat <- data.frame(
    id = c(1, 1, 1, 1, 2, 2, 2),
    time = c("pre", "pre", "post", "post", "pre", "pre", "post"),
    rater = c("r1", "r2", "r1", "r2", "r1", "r2", "r1"),
    y = c(1, 2, 3, 4, 5, 6, 7)
  )

  expect_error(
    babebi:::prepare_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      time_order = c("pre", "post"),
      rater_order = c("r1", "r2")
    ),
    "Each subject must have exactly one observation"
  )
})

library(testthat)
library(babebi)

test_that("extract_design returns the expected structure on valid standardised data", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  dat_std <- babebi:::prepare_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2")
  )

  out <- babebi:::extract_design(dat_std)

  expect_type(out, "list")
  expect_identical(
    names(out),
    c("N", "delta", "bias_r2", "time_levels", "rater_levels")
  )
  expect_equal(out$N, 4)
  expect_true(is.numeric(out$delta))
  expect_length(out$delta, 1)
  expect_true(is.numeric(out$bias_r2))
  expect_length(out$bias_r2, 1)
  expect_identical(out$time_levels, c("pre", "post"))
  expect_identical(out$rater_levels, c("r1", "r2"))
})

test_that("extract_design computes expected delta and bias_r2 values", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  dat_std <- babebi:::prepare_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2")
  )

  out <- babebi:::extract_design(dat_std)

  expect_equal(out$delta, 0.675, tolerance = 1e-10)
  expect_equal(out$bias_r2, 0.175, tolerance = 1e-10)
})

test_that("extract_design fails when required columns are missing", {
  bad_dat <- data.frame(
    id = factor(c(1, 1, 2, 2)),
    time = factor(c("pre", "post", "pre", "post")),
    y = c(1, 2, 3, 4)
  )

  expect_error(
    babebi:::extract_design(bad_dat),
    "`data` must contain the columns"
  )
})

test_that("extract_design fails when time is not a factor", {
  bad_dat <- data.frame(
    id = factor(rep(1:2, each = 4)),
    time = c("pre", "pre", "post", "post", "pre", "pre", "post", "post"),
    rater = factor(rep(c("r1", "r2"), times = 4)),
    y = c(1, 2, 3, 4, 5, 6, 7, 8)
  )

  expect_error(
    babebi:::extract_design(bad_dat),
    "`time` must be a factor with exactly two levels"
  )
})

test_that("extract_design fails when rater is not a factor", {
  bad_dat <- data.frame(
    id = factor(rep(1:2, each = 4)),
    time = factor(rep(c("pre", "pre", "post", "post"), times = 2)),
    rater = c("r1", "r2", "r1", "r2", "r1", "r2", "r1", "r2"),
    y = c(1, 2, 3, 4, 5, 6, 7, 8)
  )

  expect_error(
    babebi:::extract_design(bad_dat),
    "`rater` must be a factor with exactly two levels"
  )
})

test_that("extract_design fails when time does not have exactly two levels", {
  bad_dat <- data.frame(
    id = factor(c(1, 1, 1, 2, 2, 2)),
    time = factor(c("pre", "mid", "post", "pre", "mid", "post")),
    rater = factor(c("r1", "r1", "r1", "r2", "r2", "r2")),
    y = c(1, 2, 3, 4, 5, 6)
  )

  expect_error(
    babebi:::extract_design(bad_dat),
    "`time` must be a factor with exactly two levels"
  )
})

test_that("extract_design fails when rater does not have exactly two levels", {
  bad_dat <- data.frame(
    id = factor(c(1, 1, 1, 2, 2, 2)),
    time = factor(c("pre", "pre", "post", "pre", "pre", "post")),
    rater = factor(c("r1", "r2", "r3", "r1", "r2", "r3")),
    y = c(1, 2, 3, 4, 5, 6)
  )

  expect_error(
    babebi:::extract_design(bad_dat),
    "`rater` must be a factor with exactly two levels"
  )
})

test_that("extract_design fails when outcome is not numeric", {
  bad_dat <- data.frame(
    id = factor(rep(1:2, each = 4)),
    time = factor(rep(c("pre", "pre", "post", "post"), times = 2)),
    rater = factor(rep(c("r1", "r2"), times = 4)),
    y = as.character(1:8)
  )

  expect_error(
    babebi:::extract_design(bad_dat),
    "`y` must be numeric"
  )
})

test_that("extract_design fails when reshaped pre-post data are incomplete", {
  dat <- data.frame(
    id = factor(c(1, 1, 1, 2, 2, 2)),
    time = factor(c("pre", "pre", "post", "pre", "pre", "post"),
                  levels = c("pre", "post")),
    rater = factor(c("r1", "r2", "r1", "r1", "r2", "r1"),
                   levels = c("r1", "r2")),
    y = c(1, 2, 3, 4, 5, 6)
  )

  expect_error(
    babebi:::extract_design(dat),
    "Incomplete pre-post data detected after reshaping by subject and rater"
  )
})

#' Print a babebi fit object
#'
#' @param x An object of class `babebi_fit`.
#' @param digits Number of digits to print.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#' @export
print.babebi_fit <- function(x, digits = 3, ...) {

  if (!inherits(x, "babebi_fit")) {
    stop("`x` must be an object of class 'babebi_fit'.", call. = FALSE)
  }

  fmt <- function(z) {
    ifelse(is.finite(z), formatC(z, digits = digits, format = "f"), NA_character_)
  }

  logbf <- if (is.finite(x$bf10) && x$bf10 > 0) log(x$bf10) else NA_real_

  cat("\n")
  cat("babebi model fit\n")
  cat("----------------\n")
  cat(sprintf("beta (pre-post, ref rater): %s\n", fmt(x$beta_hat)))
  cat(sprintf("gamma (rater discrepancy): %s\n", fmt(x$gamma_hat)))
  cat(sprintf("95%% CI for beta: [%s, %s]\n", fmt(x$ci_low), fmt(x$ci_high)))
  cat(sprintf("Pr(beta > 0): %s\n", fmt(x$p_pos)))
  cat(sprintf("BF10 (BIC): %s\n", fmt(x$bf10)))
  cat(sprintf("log(BF10): %s\n", fmt(logbf)))

  if (!is.null(x$design)) {
    cat("\n")
    cat("Calibrated design\n")
    cat("-----------------\n")
    cat(sprintf("N: %d\n", as.integer(x$design$N)))
    cat(sprintf("delta: %s\n", fmt(x$design$delta)))
    cat(sprintf("bias_r2: %s\n", fmt(x$design$bias_r2)))
  }

  invisible(x)
}


#' Plot a babebi fit object
#'
#' @param x An object of class `babebi_fit`.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return No return value, called for its side effects.
#' @export
plot.babebi_fit <- function(x, ...) {

  if (!inherits(x, "babebi_fit")) {
    stop("`x` must be an object of class 'babebi_fit'.", call. = FALSE)
  }

  if (is.null(x$posterior_draws) ||
      !all(c("beta", "gamma") %in% names(x$posterior_draws))) {
    stop("`x$posterior_draws` must contain columns `beta` and `gamma`.", call. = FALSE)
  }

  beta_draws <- x$posterior_draws$beta

  if (length(beta_draws) < 2) {
    stop("Posterior draws are insufficient for density estimation.", call. = FALSE)
  }

  v <- stats::var(beta_draws)

  if (!is.finite(v) || v < .Machine$double.eps) {
    stop("Posterior draws have insufficient variability for density estimation.", call. = FALSE)
  }

  dens <- stats::density(beta_draws)

  graphics::plot(
    dens,
    main = "Posterior distribution of beta",
    xlab = expression(beta),
    ylab = "Density",
    ...
  )

  graphics::abline(v = x$ci_low, lty = 2)
  graphics::abline(v = x$ci_high, lty = 2)
  graphics::abline(v = 0, lty = 3)
  graphics::abline(v = x$beta_hat, lwd = 2)
}


#' Summarise a babebi fit object
#'
#' @param object An object of class `babebi_fit`.
#' @param digits Number of digits to print.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return An object of class `summary.babebi_fit`.
#' @export
summary.babebi_fit <- function(object, digits = 3, ...) {

  if (!inherits(object, "babebi_fit")) {
    stop("`object` must be an object of class 'babebi_fit'.", call. = FALSE)
  }

  logbf <- if (is.finite(object$bf10) && object$bf10 > 0) log(object$bf10) else NA_real_

  out <- list(
    call = match.call(),
    coefficients = data.frame(
      parameter = c("beta", "gamma"),
      estimate = c(object$beta_hat, object$gamma_hat)
    ),
    beta_interval = data.frame(
      parameter = "beta",
      ci_low = object$ci_low,
      ci_high = object$ci_high,
      p_pos = object$p_pos
    ),
    evidence = data.frame(
      bf10 = object$bf10,
      log_bf10 = logbf
    ),
    design = object$design,
    digits = digits
  )

  class(out) <- "summary.babebi_fit"
  out
}


#' Print a summary of a babebi fit object
#'
#' @param x An object of class `summary.babebi_fit`.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#' @export
print.summary.babebi_fit <- function(x, ...) {

  if (!inherits(x, "summary.babebi_fit")) {
    stop("`x` must be an object of class 'summary.babebi_fit'.", call. = FALSE)
  }

  digits <- x$digits

  fmt <- function(z) {
    ifelse(is.finite(z), formatC(z, digits = digits, format = "f"), NA_character_)
  }

  cat("\n")
  cat("Summary of babebi model fit\n")
  cat("---------------------------\n")

  coef_tab <- x$coefficients
  coef_tab$estimate <- fmt(coef_tab$estimate)
  print(coef_tab, row.names = FALSE)

  cat("\n")
  int_tab <- x$beta_interval
  int_tab$ci_low <- fmt(int_tab$ci_low)
  int_tab$ci_high <- fmt(int_tab$ci_high)
  int_tab$p_pos <- fmt(int_tab$p_pos)
  print(int_tab, row.names = FALSE)

  cat("\n")
  ev_tab <- x$evidence
  ev_tab$bf10 <- fmt(ev_tab$bf10)
  ev_tab$log_bf10 <- fmt(ev_tab$log_bf10)
  print(ev_tab, row.names = FALSE)

  if (!is.null(x$design)) {
    cat("\n")
    des_tab <- data.frame(
      N = as.integer(x$design$N),
      delta = fmt(x$design$delta),
      bias_r2 = fmt(x$design$bias_r2)
    )
    print(des_tab, row.names = FALSE)
  }

  invisible(x)
}


#' Print a babebi validation object
#'
#' @param x An object of class `babebi_validation`.
#' @param digits Number of digits to print.
#' @param ... Further arguments passed to or from other methods.
#'
#' @return The input object, invisibly.
#' @export
print.babebi_validation <- function(x, digits = 3, ...) {

  if (!inherits(x, "babebi_validation")) {
    stop("`x` must be an object of class 'babebi_validation'.", call. = FALSE)
  }

  if (!is.data.frame(x) || nrow(x) != 1L) {
    stop("`x` must be a one-row data frame.", call. = FALSE)
  }

  fmt <- function(z) {
    ifelse(is.finite(z), formatC(z, digits = digits, format = "f"), NA_character_)
  }

  bf_val <- x$bf_mean[1]
  logbf <- if (is.finite(bf_val) && bf_val > 0) log(bf_val) else NA_real_

  cat("\n")
  cat("babebi Monte Carlo validation\n")
  cat("-----------------------------\n")
  cat(sprintf("N: %d\n", as.integer(x$N[1])))
  cat(sprintf("delta: %s\n", fmt(x$delta[1])))
  cat(sprintf("bias_r2: %s\n", fmt(x$bias_r2[1])))
  cat(sprintf("R: %d\n", as.integer(x$R[1])))

  cat("\n")
  cat(sprintf("bias: %s\n", fmt(x$bias[1])))
  cat(sprintf("RMSE: %s\n", fmt(x$rmse[1])))
  cat(sprintf("coverage: %s\n", fmt(x$coverage[1])))
  cat(sprintf("mean Pr(beta > 0): %s\n", fmt(x$ppos_mean[1])))
  cat(sprintf("mean BF10: %s\n", fmt(bf_val)))
  cat(sprintf("log(mean BF10): %s\n", fmt(logbf)))

  design <- attr(x, "calibrated_design")

  if (!is.null(design)) {
    cat("\n")
    cat("Calibrated design from observed data\n")
    cat("------------------------------------\n")
    cat(sprintf("N: %d\n", as.integer(design$N)))
    cat(sprintf("delta: %s\n", fmt(design$delta)))
    cat(sprintf("bias_r2: %s\n", fmt(design$bias_r2)))
  }

  invisible(x)
}

library(testthat)
library(babebi)

test_that("print.babebi_fit returns the object invisibly", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 200,
    seed = 123
  )

  expect_invisible(print(fit))
})

test_that("summary.babebi_fit returns a summary object with expected structure", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 200,
    seed = 123
  )

  s <- summary(fit)

  expect_s3_class(s, "summary.babebi_fit")
  expect_identical(
    names(s),
    c("call", "coefficients", "beta_interval", "evidence", "design", "digits")
  )
  expect_s3_class(s$coefficients, "data.frame")
  expect_s3_class(s$beta_interval, "data.frame")
  expect_s3_class(s$evidence, "data.frame")
})

test_that("print.summary.babebi_fit returns the summary object invisibly", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 200,
    seed = 123
  )

  s <- summary(fit)

  expect_invisible(print(s))
})

test_that("plot.babebi_fit runs without error on valid posterior draws", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 200,
    seed = 123
  )

  expect_no_error(plot(fit))
})

test_that("plot.babebi_fit fails when posterior draws are invalid", {
  bad_fit <- list(
    posterior_draws = data.frame(beta = rep(1, 10), gamma = rep(0, 10)),
    ci_low = 1,
    ci_high = 1,
    beta_hat = 1
  )
  class(bad_fit) <- "babebi_fit"

  expect_error(
    plot(bad_fit),
    "Posterior draws have insufficient variability for density estimation"
  )
})

test_that("print.babebi_validation returns the object invisibly", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  mc <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 25,
    seed = 123
  )

  expect_invisible(print(mc))
})

test_that("print methods fail on wrong object classes", {
  expect_error(
    print.babebi_fit(list()),
    "`x` must be an object of class 'babebi_fit'"
  )

  expect_error(
    summary.babebi_fit(list()),
    "`object` must be an object of class 'babebi_fit'"
  )

  expect_error(
    print.summary.babebi_fit(list()),
    "`x` must be an object of class 'summary.babebi_fit'"
  )

  expect_error(
    plot.babebi_fit(list()),
    "`x` must be an object of class 'babebi_fit'"
  )

  expect_error(
    print.babebi_validation(list()),
    "`x` must be an object of class 'babebi_validation'"
  )
})

#' Run one Monte Carlo validation cell
#'
#' Internal helper used to evaluate the analytical babebi model under a single
#' simulation condition defined by sample size, pre-post effect, and rater bias.
#'
#' The current implementation simulates pre-post difference scores directly for
#' the reference rater and the second rater, then applies the same analytical
#' linear-model logic used in `fit_model()`.
#'
#' @param N Integer sample size.
#' @param delta Numeric pre-post effect for the reference rater.
#' @param bias_r2 Numeric additive discrepancy for the second rater.
#' @param R Integer number of Monte Carlo repetitions.
#'
#' @return A one-row data frame with columns `N`, `delta`, `bias_r2`, `bias`,
#'   `rmse`, `coverage`, `ppos_mean`, and `bf_mean`.
#'
#' @keywords internal
run_cell <- function(N, delta, bias_r2, R) {
  
  if (!is.numeric(N) || length(N) != 1L || is.na(N) || N <= 1 || N %% 1 != 0) {
    stop("`N` must be a single integer greater than 1.", call. = FALSE)
  }
  
  if (!is.numeric(delta) || length(delta) != 1L || is.na(delta)) {
    stop("`delta` must be a single numeric value.", call. = FALSE)
  }
  
  if (!is.numeric(bias_r2) || length(bias_r2) != 1L || is.na(bias_r2)) {
    stop("`bias_r2` must be a single numeric value.", call. = FALSE)
  }
  
  if (!is.numeric(R) || length(R) != 1L || is.na(R) || R <= 0 || R %% 1 != 0) {
    stop("`R` must be a positive integer.", call. = FALSE)
  }
  
  beta_hat <- numeric(R)
  covered <- logical(R)
  ppos <- numeric(R)
  bf10 <- numeric(R)
  
  for (r in seq_len(R)) {
    d_ref <- stats::rnorm(n = N, mean = delta, sd = 1)
    d_r2 <- stats::rnorm(n = N, mean = delta + bias_r2, sd = 1)
    
    y <- c(d_ref, d_r2)
    X <- cbind(
      Intercept = 1,
      r2 = c(rep(0, N), rep(1, N))
    )
    
    XtX <- crossprod(X)
    XtY <- crossprod(X, y)
    
    XtX_inv <- solve(XtX)
    bh <- as.vector(XtX_inv %*% XtY)
    
    resid <- y - as.vector(X %*% bh)
    n_obs <- nrow(X)
    p <- ncol(X)
    
    df_resid <- n_obs - p
    sigma2_hat <- sum(resid^2) / df_resid
    V_beta <- sigma2_hat * XtX_inv
    
    eps <- 1e-10
    diag(V_beta) <- diag(V_beta) + eps
    
    se_beta <- sqrt(V_beta[1, 1])
    ci_low <- bh[1] - stats::qnorm(0.975) * se_beta
    ci_high <- bh[1] + stats::qnorm(0.975) * se_beta
    
    beta_hat[r] <- bh[1]
    covered[r] <- (ci_low <= delta) && (delta <= ci_high)
    ppos[r] <- 1 - stats::pnorm(0, mean = bh[1], sd = se_beta)
    
    k_alt <- 2
    rss_alt <- max(sum(resid^2), .Machine$double.eps)
    bic_alt <- n_obs * log(rss_alt / n_obs) + k_alt * log(n_obs)
    
    X0 <- X[, "r2", drop = FALSE]
    XtX0 <- crossprod(X0)
    
    if (as.numeric(XtX0) < .Machine$double.eps) {
      bf10[r] <- NA_real_
    } else {
      bh0 <- as.numeric(crossprod(X0, y) / XtX0)
      resid0 <- y - X0 %*% bh0
      
      k_null <- 1
      rss_null <- max(sum(resid0^2), .Machine$double.eps)
      bic_null <- n_obs * log(rss_null / n_obs) + k_null * log(n_obs)
      
      bf10[r] <- exp((bic_null - bic_alt) / 2)
    }
  }
  
  data.frame(
    N = as.integer(N),
    delta = as.numeric(delta),
    bias_r2 = as.numeric(bias_r2),
    bias = mean(beta_hat - delta),
    rmse = sqrt(mean((beta_hat - delta)^2)),
    coverage = mean(covered),
    ppos_mean = mean(ppos),
    bf_mean = mean(bf10, na.rm = TRUE)
  )
}

#' Run Monte Carlo calibration from user-supplied data
#'
#' Uses observed data to estimate the sample size, mean pre-post change,
#' and mean additive discrepancy of the second rater for the Monte Carlo
#' engine, then runs a single Monte Carlo validation cell.
#'
#' @param data A data frame in long format.
#' @param id Unquoted column name identifying subjects or a character string.
#' @param time Unquoted column name identifying time points or a character string.
#' @param rater Unquoted column name identifying raters or a character string.
#' @param outcome Unquoted column name containing the observed score or a
#'   character string.
#' @param time_order Optional character vector of length 2 giving the
#'   order of the time factor: baseline first, follow-up second.
#' @param rater_order Optional character vector of length 2 giving the
#'   order of the rater factor: reference rater first, second rater second.
#' @param R Number of Monte Carlo repetitions.
#' @param N Optional override for the sample size used in the simulation.
#'   Defaults to the observed number of subjects.
#' @param delta Optional override for the pre-post effect used in the
#'   simulation. Defaults to the value estimated from the data.
#' @param bias_r2 Optional override for the additive bias of the second
#'   rater. Defaults to the value estimated from the data.
#' @param seed Optional integer seed used to initialise the random-number
#'   generator for simulation.
#'
#' @return A one-row data frame of class `babebi_validation` with Monte Carlo
#'   performance summaries. The attributes `calibrated_design` and `call`
#'   store the design estimated from the observed data and the original call.
#'
#' @examples
#' dat <- data.frame(
#'   id = rep(1:4, each = 4),
#'   time = rep(c("pre", "pre", "post", "post"), times = 4),
#'   rater = rep(c("r1", "r2"), times = 8),
#'   y = c(
#'     3.0, 3.2, 3.8, 4.0,
#'     2.9, 3.1, 3.5, 3.7,
#'     3.4, 3.6, 4.0, 4.1,
#'     3.1, 3.3, 3.9, 4.0
#'   )
#' )
#'
#' mc <- montecarlo_from_data(
#'   data = dat,
#'   id = id,
#'   time = time,
#'   rater = rater,
#'   outcome = y,
#'   time_order = c("pre", "post"),
#'   rater_order = c("r1", "r2"),
#'   R = 100,
#'   seed = 123
#' )
#'
#' print(mc)
#'
#' @export
montecarlo_from_data <- function(data,
                                 id,
                                 time,
                                 rater,
                                 outcome,
                                 time_order = NULL,
                                 rater_order = NULL,
                                 R = 1000,
                                 N = NULL,
                                 delta = NULL,
                                 bias_r2 = NULL,
                                 seed = NULL) {

  id_name <- resolve_colname(substitute(id), parent.frame())
  time_name <- resolve_colname(substitute(time), parent.frame())
  rater_name <- resolve_colname(substitute(rater), parent.frame())
  outcome_name <- resolve_colname(substitute(outcome), parent.frame())

  if (!is.numeric(R) || length(R) != 1L || is.na(R) || R <= 0 || R %% 1 != 0) {
    stop("`R` must be a positive integer.", call. = FALSE)
  }

  if (!is.null(N) &&
      (!is.numeric(N) || length(N) != 1L || is.na(N) ||
       N <= 1 || N %% 1 != 0)) {
    stop("`N` must be a single integer greater than 1.", call. = FALSE)
  }

  if (!is.null(delta) &&
      (!is.numeric(delta) || length(delta) != 1L || is.na(delta))) {
    stop("`delta` must be a single numeric value.", call. = FALSE)
  }

  if (!is.null(bias_r2) &&
      (!is.numeric(bias_r2) || length(bias_r2) != 1L || is.na(bias_r2))) {
    stop("`bias_r2` must be a single numeric value.", call. = FALSE)
  }

  if (!is.null(seed) &&
      (!is.numeric(seed) || length(seed) != 1L || is.na(seed) ||
       seed %% 1 != 0)) {
    stop("`seed` must be a single integer value.", call. = FALSE)
  }

  if (!is.null(seed)) {
    set.seed(as.integer(seed))
  }

  dat <- prepare_data(
    data = data,
    id = id_name,
    time = time_name,
    rater = rater_name,
    outcome = outcome_name,
    time_order = time_order,
    rater_order = rater_order
  )

  design <- extract_design(dat)

  sim_N <- if (is.null(N)) design$N else as.integer(N)
  sim_delta <- if (is.null(delta)) design$delta else as.numeric(delta)
  sim_bias_r2 <- if (is.null(bias_r2)) design$bias_r2 else as.numeric(bias_r2)

  res <- run_cell(
    N = sim_N,
    delta = sim_delta,
    bias_r2 = sim_bias_r2,
    R = as.integer(R)
  )

  res <- as.data.frame(res)
  res$R <- as.integer(R)

  wanted <- c("N", "delta", "bias_r2", "R",
              "bias", "rmse", "coverage", "ppos_mean", "bf_mean")

  missing <- setdiff(wanted, names(res))
  if (length(missing) > 0) {
    stop(
      "Monte Carlo output is missing required fields: ",
      paste(missing, collapse = ", "),
      call. = FALSE
    )
  }

  res <- res[, wanted, drop = FALSE]

  attr(res, "calibrated_design") <- design
  attr(res, "call") <- match.call()

  class(res) <- c("babebi_validation", "data.frame")

  res
}

test_that("babebi validation component is coherent with direct montecarlo_from_data output", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  res <- babebi(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    validate = TRUE,
    R = 50,
    n_draws = 500,
    seed = 123
  )

  mc <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 50,
    seed = 123
  )

  expect_equal(
    unclass(res$validation),
    unclass(mc),
    ignore_attr = TRUE
  )

  expect_equal(
    attr(res$validation, "calibrated_design"),
    attr(mc, "calibrated_design")
  )
})

library(testthat)
library(babebi)

test_that("montecarlo_from_data returns a babebi_validation object with expected structure", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  mc <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 100,
    seed = 123
  )

  expect_s3_class(mc, "babebi_validation")
  expect_s3_class(mc, "data.frame")
  expect_equal(nrow(mc), 1)

  expect_identical(
    names(mc),
    c("N", "delta", "bias_r2", "R",
      "bias", "rmse", "coverage", "ppos_mean", "bf_mean")
  )

  expect_equal(mc$N, 4)
  expect_equal(mc$R, 100)

  expect_true(is.numeric(mc$delta))
  expect_true(is.numeric(mc$bias_r2))
  expect_true(is.numeric(mc$bias))
  expect_true(is.numeric(mc$rmse))
  expect_true(is.numeric(mc$coverage))
  expect_true(is.numeric(mc$ppos_mean))
  expect_true(is.numeric(mc$bf_mean))

  expect_true(!is.null(attr(mc, "calibrated_design")))
  expect_true(!is.null(attr(mc, "call")))

  design <- attr(mc, "calibrated_design")
  expect_type(design, "list")
  expect_identical(
    names(design),
    c("N", "delta", "bias_r2", "time_levels", "rater_levels")
  )
})

test_that("montecarlo_from_data returns coherent calibrated design values", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  mc <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 50,
    seed = 123
  )

  design <- attr(mc, "calibrated_design")

  expect_equal(design$N, 4)
  expect_equal(design$delta, 0.675, tolerance = 1e-10)
  expect_equal(design$bias_r2, 0.175, tolerance = 1e-10)
  expect_identical(design$time_levels, c("pre", "post"))
  expect_identical(design$rater_levels, c("r1", "r2"))
})

test_that("montecarlo_from_data respects user overrides for N, delta, and bias_r2", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  mc <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 20,
    N = 10,
    delta = 0.50,
    bias_r2 = 0.25,
    seed = 123
  )

  expect_equal(mc$N, 10)
  expect_equal(mc$delta, 0.50, tolerance = 1e-10)
  expect_equal(mc$bias_r2, 0.25, tolerance = 1e-10)

  design <- attr(mc, "calibrated_design")
  expect_equal(design$N, 4)
  expect_equal(design$delta, 0.675, tolerance = 1e-10)
  expect_equal(design$bias_r2, 0.175, tolerance = 1e-10)
})

test_that("montecarlo_from_data is reproducible when seed is fixed", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  mc1 <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 50,
    seed = 999
  )

  mc2 <- montecarlo_from_data(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 50,
    seed = 999
  )

  expect_equal(mc1, mc2)
  expect_equal(attr(mc1, "calibrated_design"), attr(mc2, "calibrated_design"))
})

test_that("montecarlo_from_data accepts character column names", {
  dat <- data.frame(
    subj = rep(1:4, each = 4),
    occasion = rep(c("pre", "pre", "post", "post"), times = 4),
    judge = rep(c("r1", "r2"), times = 8),
    score = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  mc <- montecarlo_from_data(
    data = dat,
    id = "subj",
    time = "occasion",
    rater = "judge",
    outcome = "score",
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    R = 25,
    seed = 123
  )

  expect_s3_class(mc, "babebi_validation")
  expect_equal(nrow(mc), 1)
  expect_equal(mc$R, 25)
})

test_that("montecarlo_from_data fails when R is not a positive integer", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      R = 0
    ),
    "`R` must be a positive integer"
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      R = 10.5
    ),
    "`R` must be a positive integer"
  )
})

test_that("montecarlo_from_data fails when N is not a valid integer greater than 1", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      N = 1
    ),
    "`N` must be a single integer greater than 1"
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      N = 2.5
    ),
    "`N` must be a single integer greater than 1"
  )
})

test_that("montecarlo_from_data fails when delta is not a single numeric value", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      delta = c(0.1, 0.2)
    ),
    "`delta` must be a single numeric value"
  )
})

test_that("montecarlo_from_data fails when bias_r2 is not a single numeric value", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      bias_r2 = c(0.1, 0.2)
    ),
    "`bias_r2` must be a single numeric value"
  )
})

test_that("montecarlo_from_data fails when seed is not an integer", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    montecarlo_from_data(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      seed = 1.5
    ),
    "`seed` must be a single integer value"
  )
})

#' Run the babebi workflow on user-supplied data
#'
#' Fits the babebi model to observed data and, optionally, performs Monte Carlo
#' validation calibrated on the observed design.
#'
#' @param data A data frame in long format.
#' @param id Unquoted column name identifying subjects or a character string.
#' @param time Unquoted column name identifying time points or a character string.
#' @param rater Unquoted column name identifying raters or a character string.
#' @param outcome Unquoted column name containing the observed score or a
#'   character string.
#' @param time_order Optional character vector of length 2 giving the
#'   order of the time factor: baseline first, follow-up second.
#' @param rater_order Optional character vector of length 2 giving the
#'   order of the rater factor: reference rater first, second rater second.
#' @param validate Logical; if `TRUE`, run Monte Carlo validation after model fitting.
#' @param R Number of Monte Carlo repetitions when `validate = TRUE`.
#' @param N Optional override for the sample size used in validation.
#' @param delta Optional override for the pre-post effect used in validation.
#' @param bias_r2 Optional override for the additive bias of the second rater
#'   used in validation.
#' @param n_draws Number of posterior draws used in model fitting.
#' @param seed Optional integer seed for reproducibility.
#'
#' @return A named list containing:
#' \describe{
#'   \item{fit}{A `babebi_fit` object returned by `fit_model()`.}
#'   \item{validation}{A `babebi_validation` object returned by
#'   `montecarlo_from_data()` when `validate = TRUE`, otherwise `NULL`.}
#'   \item{call}{The original function call.}
#' }
#'
#' @examples
#' dat <- data.frame(
#'   id = rep(1:4, each = 4),
#'   time = rep(c("pre", "pre", "post", "post"), times = 4),
#'   rater = rep(c("r1", "r2"), times = 8),
#'   y = c(
#'     3.0, 3.2, 3.8, 4.0,
#'     2.9, 3.1, 3.5, 3.7,
#'     3.4, 3.6, 4.0, 4.1,
#'     3.1, 3.3, 3.9, 4.0
#'   )
#' )
#'
#' res <- babebi(
#'   data = dat,
#'   id = id,
#'   time = time,
#'   rater = rater,
#'   outcome = y,
#'   time_order = c("pre", "post"),
#'   rater_order = c("r1", "r2"),
#'   validate = TRUE,
#'   R = 100,
#'   n_draws = 500,
#'   seed = 123
#' )
#'
#' print(res$fit)
#' summary(res$fit)
#' plot(res$fit)
#' print(res$validation)
#'
#' @export
babebi <- function(data,
                   id,
                   time,
                   rater,
                   outcome,
                   time_order = NULL,
                   rater_order = NULL,
                   validate = TRUE,
                   R = 1000,
                   N = NULL,
                   delta = NULL,
                   bias_r2 = NULL,
                   n_draws = 4000,
                   seed = NULL) {

  id_name <- resolve_colname(substitute(id), parent.frame())
  time_name <- resolve_colname(substitute(time), parent.frame())
  rater_name <- resolve_colname(substitute(rater), parent.frame())
  outcome_name <- resolve_colname(substitute(outcome), parent.frame())

  if (!is.logical(validate) || length(validate) != 1L || is.na(validate)) {
    stop("`validate` must be a single TRUE or FALSE value.", call. = FALSE)
  }

  fit <- fit_model(
    data = data,
    id = id_name,
    time = time_name,
    rater = rater_name,
    outcome = outcome_name,
    time_order = time_order,
    rater_order = rater_order,
    n_draws = n_draws,
    seed = seed
  )

  validation <- NULL

  if (isTRUE(validate)) {
    validation <- montecarlo_from_data(
      data = data,
      id = id_name,
      time = time_name,
      rater = rater_name,
      outcome = outcome_name,
      time_order = time_order,
      rater_order = rater_order,
      R = R,
      N = N,
      delta = delta,
      bias_r2 = bias_r2,
      seed = seed
    )
  }

  list(
    fit = fit,
    validation = validation,
    call = match.call()
  )
}

#' Resolve a column name supplied as symbol or character string
#'
#' Internal helper for functions that accept either unquoted column names
#' or character strings.
#'
#' @param expr A captured expression representing a column name.
#' @param env The environment in which the expression should be resolved.
#'
#' @return A character string of length 1.
#'
#' @keywords internal
resolve_colname <- function(expr, env = parent.frame()) {

  if (is.character(expr)) {
    return(as.character(expr))
  }

  if (is.symbol(expr)) {
    nm <- as.character(expr)

    if (exists(nm, envir = env, inherits = TRUE)) {
      val <- get(nm, envir = env, inherits = TRUE)

      if (is.character(val) && length(val) == 1L) {
        return(val)
      }
    }

    return(nm)
  }

  deparse(expr)
}

#' Fit the babebi model to observed data
#'
#' Fits the current babebi model to a fully observed 2 (time) x 2 (rater)
#' repeated-measures dataset. Estimation is based on difference scores and a
#' linear model including a rater indicator as covariate.
#'
#' Posterior draws are obtained from a large-sample normal approximation to
#' the posterior of the regression coefficients under an improper flat prior.
#'
#' @param data A data frame in long format.
#' @param id Unquoted column name identifying subjects or a character string.
#' @param time Unquoted column name identifying time points or a character string.
#' @param rater Unquoted column name identifying raters or a character string.
#' @param outcome Unquoted column name containing the observed score or a
#'   character string.
#' @param time_order Optional character vector of length 2 giving the
#'   order of the time factor: baseline first, follow-up second.
#' @param rater_order Optional character vector of length 2 giving the
#'   order of the rater factor: reference rater first, second rater second.
#' @param n_draws Number of posterior draws used for approximation.
#' @param seed Optional integer seed used to initialise the random-number
#'   generator for posterior sampling.
#'
#' @return A named list of class `babebi_fit`.
#'
#' @examples
#' dat <- data.frame(
#'   id = rep(1:4, each = 4),
#'   time = rep(c("pre", "pre", "post", "post"), times = 4),
#'   rater = rep(c("r1", "r2"), times = 8),
#'   y = c(
#'     3.0, 3.2, 3.8, 4.0,
#'     2.9, 3.1, 3.5, 3.7,
#'     3.4, 3.6, 4.0, 4.1,
#'     3.1, 3.3, 3.9, 4.0
#'   )
#' )
#'
#' fit <- fit_model(
#'   data = dat,
#'   id = id,
#'   time = time,
#'   rater = rater,
#'   outcome = y,
#'   time_order = c("pre", "post"),
#'   rater_order = c("r1", "r2"),
#'   n_draws = 500,
#'   seed = 123
#' )
#'
#' print(fit)
#' summary(fit)
#' plot(fit)
#'
#' @export
fit_model <- function(data,
                      id,
                      time,
                      rater,
                      outcome,
                      time_order = NULL,
                      rater_order = NULL,
                      n_draws = 4000,
                      seed = NULL) {

  id_name <- resolve_colname(substitute(id), parent.frame())
  time_name <- resolve_colname(substitute(time), parent.frame())
  rater_name <- resolve_colname(substitute(rater), parent.frame())
  outcome_name <- resolve_colname(substitute(outcome), parent.frame())

  if (!is.numeric(n_draws) || length(n_draws) != 1L || is.na(n_draws) ||
      n_draws <= 0 || n_draws %% 1 != 0) {
    stop("`n_draws` must be a positive integer.", call. = FALSE)
  }

  if (!is.null(seed) &&
      (!is.numeric(seed) || length(seed) != 1L || is.na(seed) ||
       seed %% 1 != 0)) {
    stop("`seed` must be a single integer value.", call. = FALSE)
  }

  if (!is.null(seed)) {
    set.seed(as.integer(seed))
  }

  dat <- prepare_data(
    data = data,
    id = id_name,
    time = time_name,
    rater = rater_name,
    outcome = outcome_name,
    time_order = time_order,
    rater_order = rater_order
  )

  design <- extract_design(dat)

  time_levels <- levels(dat$time)
  rater_levels <- levels(dat$rater)

  baseline <- time_levels[1]
  followup <- time_levels[2]
  second_rater <- rater_levels[2]

  wide_time <- stats::reshape(
    dat,
    idvar = c("id", "rater"),
    timevar = "time",
    direction = "wide"
  )

  base_col <- make.names(paste0("y.", baseline))
  follow_col <- make.names(paste0("y.", followup))

  if (!all(c(base_col, follow_col) %in% names(wide_time))) {
    stop("Failed to reconstruct pre-post outcome columns after reshaping.", call. = FALSE)
  }

  y_base <- wide_time[[base_col]]
  y_follow <- wide_time[[follow_col]]

  if (anyNA(y_base) || anyNA(y_follow)) {
    stop("Incomplete pre-post data detected after reshaping.", call. = FALSE)
  }

  wide_time$d <- y_follow - y_base
  wide_time$r2 <- as.numeric(wide_time$rater == second_rater)

  X <- cbind(Intercept = 1, r2 = wide_time$r2)
  y <- wide_time$d

  XtX <- crossprod(X)
  XtY <- crossprod(X, y)

  XtX_inv <- solve(XtX)
  bh <- as.vector(XtX_inv %*% XtY)

  resid <- y - as.vector(X %*% bh)
  n <- nrow(X)
  p <- ncol(X)

  df_resid <- n - p
  if (df_resid <= 0) {
    stop("Insufficient degrees of freedom for variance estimation.", call. = FALSE)
  }

  sigma2_hat <- sum(resid^2) / df_resid
  V_beta <- sigma2_hat * XtX_inv

  eps <- 1e-10
  diag(V_beta) <- diag(V_beta) + eps

  L <- chol(V_beta)
  Z <- matrix(stats::rnorm(as.integer(n_draws) * p), ncol = p)
  theta_draws <- sweep(Z %*% t(L), 2, bh, `+`)

  beta_draws <- theta_draws[, 1]
  gamma_draws <- theta_draws[, 2]

  ci <- stats::quantile(beta_draws, c(0.025, 0.975), names = FALSE)
  p_pos <- mean(beta_draws > 0)

  k_alt <- 2
  rss_alt <- max(sum(resid^2), .Machine$double.eps)
  bic_alt <- n * log(rss_alt / n) + k_alt * log(n)

  X0 <- X[, "r2", drop = FALSE]
  XtX0 <- crossprod(X0)

  if (as.numeric(XtX0) < .Machine$double.eps) {
    stop("Degenerate design: rater indicator has no variability.", call. = FALSE)
  }

  bh0 <- as.numeric(crossprod(X0, y) / XtX0)
  resid0 <- y - X0 %*% bh0

  k_null <- 1
  rss_null <- max(sum(resid0^2), .Machine$double.eps)
  bic_null <- n * log(rss_null / n) + k_null * log(n)

  bf10 <- exp((bic_null - bic_alt) / 2)

  res <- list(
    beta_hat = unname(bh[1]),
    gamma_hat = unname(bh[2]),
    ci_low = ci[1],
    ci_high = ci[2],
    p_pos = p_pos,
    bf10 = bf10,
    posterior_draws = data.frame(
      beta = as.numeric(beta_draws),
      gamma = as.numeric(gamma_draws)
    ),
    design = design
  )

  class(res) <- "babebi_fit"

  res
}

library(testthat)
library(babebi)

test_that("fit_model returns a babebi_fit object with expected components", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 500,
    seed = 123
  )

  expect_s3_class(fit, "babebi_fit")
  expect_identical(
    names(fit),
    c(
      "beta_hat",
      "gamma_hat",
      "ci_low",
      "ci_high",
      "p_pos",
      "bf10",
      "posterior_draws",
      "design"
    )
  )

  expect_true(is.numeric(fit$beta_hat))
  expect_length(fit$beta_hat, 1)

  expect_true(is.numeric(fit$gamma_hat))
  expect_length(fit$gamma_hat, 1)

  expect_true(is.numeric(fit$ci_low))
  expect_length(fit$ci_low, 1)

  expect_true(is.numeric(fit$ci_high))
  expect_length(fit$ci_high, 1)

  expect_true(is.numeric(fit$p_pos))
  expect_length(fit$p_pos, 1)

  expect_true(is.numeric(fit$bf10))
  expect_length(fit$bf10, 1)

  expect_s3_class(fit$posterior_draws, "data.frame")
  expect_identical(names(fit$posterior_draws), c("beta", "gamma"))
  expect_equal(nrow(fit$posterior_draws), 500)

  expect_type(fit$design, "list")
  expect_identical(
    names(fit$design),
    c("N", "delta", "bias_r2", "time_levels", "rater_levels")
  )
})

test_that("fit_model returns numerically coherent estimates on the example dataset", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 1000,
    seed = 123
  )

  expect_equal(fit$beta_hat, 0.70, tolerance = 1e-10)
  expect_equal(fit$gamma_hat, -0.05, tolerance = 1e-10)

  expect_true(fit$ci_low < fit$ci_high)
  expect_true(fit$p_pos >= 0 && fit$p_pos <= 1)
  expect_true(is.finite(fit$bf10))
  expect_true(fit$bf10 >= 0)

  expect_equal(fit$design$N, 4)
  expect_equal(fit$design$delta, 0.675, tolerance = 1e-10)
  expect_equal(fit$design$bias_r2, 0.175, tolerance = 1e-10)
})

test_that("fit_model is reproducible when seed is fixed", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit1 <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 500,
    seed = 999
  )

  fit2 <- fit_model(
    data = dat,
    id = id,
    time = time,
    rater = rater,
    outcome = y,
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 500,
    seed = 999
  )

  expect_equal(fit1$beta_hat, fit2$beta_hat)
  expect_equal(fit1$gamma_hat, fit2$gamma_hat)
  expect_equal(fit1$ci_low, fit2$ci_low)
  expect_equal(fit1$ci_high, fit2$ci_high)
  expect_equal(fit1$p_pos, fit2$p_pos)
  expect_equal(fit1$bf10, fit2$bf10)
  expect_equal(fit1$posterior_draws, fit2$posterior_draws)
})

test_that("fit_model accepts character column names", {
  dat <- data.frame(
    subj = rep(1:4, each = 4),
    occasion = rep(c("pre", "pre", "post", "post"), times = 4),
    judge = rep(c("r1", "r2"), times = 8),
    score = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  fit <- fit_model(
    data = dat,
    id = "subj",
    time = "occasion",
    rater = "judge",
    outcome = "score",
    time_order = c("pre", "post"),
    rater_order = c("r1", "r2"),
    n_draws = 200,
    seed = 123
  )

  expect_s3_class(fit, "babebi_fit")
  expect_equal(nrow(fit$posterior_draws), 200)
})

test_that("fit_model fails when n_draws is not a positive integer", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    fit_model(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      n_draws = 0
    ),
    "`n_draws` must be a positive integer"
  )

  expect_error(
    fit_model(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      n_draws = 10.5
    ),
    "`n_draws` must be a positive integer"
  )
})

test_that("fit_model fails when seed is not an integer", {
  dat <- data.frame(
    id = rep(1:4, each = 4),
    time = rep(c("pre", "pre", "post", "post"), times = 4),
    rater = rep(c("r1", "r2"), times = 8),
    y = c(
      3.0, 3.2, 3.8, 4.0,
      2.9, 3.1, 3.5, 3.7,
      3.4, 3.6, 4.0, 4.1,
      3.1, 3.3, 3.9, 4.0
    )
  )

  expect_error(
    fit_model(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      seed = 1.5
    ),
    "`seed` must be a single integer value"
  )
})

test_that("fit_model fails when there are insufficient residual degrees of freedom", {
  dat <- data.frame(
    id = rep(1, each = 4),
    time = c("pre", "pre", "post", "post"),
    rater = c("r1", "r2", "r1", "r2"),
    y = c(1, 2, 3, 4)
  )

  expect_error(
    fit_model(
      data = dat,
      id = id,
      time = time,
      rater = rater,
      outcome = y,
      time_order = c("pre", "post"),
      rater_order = c("r1", "r2"),
      n_draws = 100,
      seed = 123
    ),
    "Insufficient degrees of freedom for variance estimation"
  )
})

#' Extract design parameters for Monte Carlo calibration
#'
#' Internal helper used to estimate sample size, mean pre-post change,
#' and second-rater additive discrepancy from standardised user-supplied
#' data for Monte Carlo calibration.
#'
#' @param data A data frame returned by `prepare_data`, with columns
#'   `id`, `time`, `rater`, and `y`.
#'
#' @return A named list containing:
#' \describe{
#'   \item{N}{Number of unique subjects.}
#'   \item{delta}{Estimated mean pre-post change, averaged across raters.}
#'   \item{bias_r2}{Estimated mean additive discrepancy of the second rater
#'   relative to the reference rater, averaged across subjects and time points.}
#'   \item{time_levels}{The two ordered time levels used in the data.}
#'   \item{rater_levels}{The two ordered rater levels used in the data.}
#' }
#'
#' @keywords internal
extract_design <- function(data) {

  required_cols <- c("id", "time", "rater", "y")

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame.", call. = FALSE)
  }

  if (!all(required_cols %in% names(data))) {
    stop(
      "`data` must contain the columns: ",
      paste(required_cols, collapse = ", "),
      call. = FALSE
    )
  }

  if (!is.factor(data$id)) {
    data$id <- factor(data$id)
  }
  data$id <- droplevels(data$id)

  if (!is.factor(data$time)) {
    stop("`time` must be a factor with exactly two levels.", call. = FALSE)
  }
  data$time <- droplevels(data$time)
  if (nlevels(data$time) != 2) {
    stop("`time` must be a factor with exactly two levels.", call. = FALSE)
  }

  if (!is.factor(data$rater)) {
    stop("`rater` must be a factor with exactly two levels.", call. = FALSE)
  }
  data$rater <- droplevels(data$rater)
  if (nlevels(data$rater) != 2) {
    stop("`rater` must be a factor with exactly two levels.", call. = FALSE)
  }

  if (!is.numeric(data$y)) {
    stop("`y` must be numeric.", call. = FALSE)
  }

  time_levels <- levels(data$time)
  rater_levels <- levels(data$rater)

  baseline <- time_levels[1]
  followup <- time_levels[2]
  ref_rater <- rater_levels[1]
  second_rater <- rater_levels[2]

  N <- nlevels(data$id)

  wide_time <- stats::reshape(
    data,
    idvar = c("id", "rater"),
    timevar = "time",
    direction = "wide"
  )

  base_col <- make.names(paste0("y.", baseline))
  follow_col <- make.names(paste0("y.", followup))

  if (!all(c(base_col, follow_col) %in% names(wide_time))) {
    stop(
      "Failed to reconstruct pre-post outcome columns after reshaping. Check time levels and naming consistency.",
      call. = FALSE
    )
  }

  y_base <- wide_time[[base_col]]
  y_follow <- wide_time[[follow_col]]

  if (anyNA(y_base) || anyNA(y_follow)) {
    stop(
      "Incomplete pre-post data detected after reshaping by subject and rater.",
      call. = FALSE
    )
  }

  change_scores <- y_follow - y_base
  delta <- mean(change_scores)

  wide_rater <- stats::reshape(
    data,
    idvar = c("id", "time"),
    timevar = "rater",
    direction = "wide"
  )

  ref_col <- make.names(paste0("y.", ref_rater))
  r2_col <- make.names(paste0("y.", second_rater))

  if (!all(c(ref_col, r2_col) %in% names(wide_rater))) {
    stop(
      "Failed to reconstruct rater-specific outcome columns after reshaping. Check rater levels and naming consistency.",
      call. = FALSE
    )
  }

  y_ref <- wide_rater[[ref_col]]
  y_r2 <- wide_rater[[r2_col]]

  if (anyNA(y_ref) || anyNA(y_r2)) {
    stop(
      "Incomplete rater data detected after reshaping by subject and time.",
      call. = FALSE
    )
  }

  rater_discrepancy <- y_r2 - y_ref
  bias_r2 <- mean(rater_discrepancy)

  list(
    N = N,
    delta = as.numeric(delta),
    bias_r2 = as.numeric(bias_r2),
    time_levels = time_levels,
    rater_levels = rater_levels
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_model.R
\name{fit_model}
\alias{fit_model}
\title{Fit the babebi model to observed data}
\usage{
fit_model(
  data,
  id,
  time,
  rater,
  outcome,
  time_order = NULL,
  rater_order = NULL,
  n_draws = 4000,
  seed = NULL
)
}
\arguments{
\item{data}{A data frame in long format.}

\item{id}{Unquoted column name identifying subjects or a character string.}

\item{time}{Unquoted column name identifying time points or a character string.}

\item{rater}{Unquoted column name identifying raters or a character string.}

\item{outcome}{Unquoted column name containing the observed score or a
character string.}

\item{time_order}{Optional character vector of length 2 giving the
order of the time factor: baseline first, follow-up second.}

\item{rater_order}{Optional character vector of length 2 giving the
order of the rater factor: reference rater first, second rater second.}

\item{n_draws}{Number of posterior draws used for approximation.}

\item{seed}{Optional integer seed used to initialise the random-number
generator for posterior sampling.}
}
\value{
A named list of class \code{babebi_fit}.
}
\description{
Fits the current babebi model to a fully observed 2 (time) x 2 (rater)
repeated-measures dataset. Estimation is based on difference scores and a
linear model including a rater indicator as covariate.
}
\details{
Posterior draws are obtained from a large-sample normal approximation to
the posterior of the regression coefficients under an improper flat prior.
}
\examples{
dat <- data.frame(
  id = rep(1:4, each = 4),
  time = rep(c("pre", "pre", "post", "post"), times = 4),
  rater = rep(c("r1", "r2"), times = 8),
  y = c(
    3.0, 3.2, 3.8, 4.0,
    2.9, 3.1, 3.5, 3.7,
    3.4, 3.6, 4.0, 4.1,
    3.1, 3.3, 3.9, 4.0
  )
)

fit <- fit_model(
  data = dat,
  id = id,
  time = time,
  rater = rater,
  outcome = y,
  time_order = c("pre", "post"),
  rater_order = c("r1", "r2"),
  n_draws = 500,
  seed = 123
)

print(fit)
summary(fit)
plot(fit)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{plot.babebi_fit}
\alias{plot.babebi_fit}
\title{Plot a babebi fit object}
\usage{
\method{plot}{babebi_fit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{babebi_fit}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
No return value, called for its side effects.
}
\description{
Plot a babebi fit object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/babebi.R
\name{babebi}
\alias{babebi}
\title{Run the babebi workflow on user-supplied data}
\usage{
babebi(
  data,
  id,
  time,
  rater,
  outcome,
  time_order = NULL,
  rater_order = NULL,
  validate = TRUE,
  R = 1000,
  N = NULL,
  delta = NULL,
  bias_r2 = NULL,
  n_draws = 4000,
  seed = NULL
)
}
\arguments{
\item{data}{A data frame in long format.}

\item{id}{Unquoted column name identifying subjects or a character string.}

\item{time}{Unquoted column name identifying time points or a character string.}

\item{rater}{Unquoted column name identifying raters or a character string.}

\item{outcome}{Unquoted column name containing the observed score or a
character string.}

\item{time_order}{Optional character vector of length 2 giving the
order of the time factor: baseline first, follow-up second.}

\item{rater_order}{Optional character vector of length 2 giving the
order of the rater factor: reference rater first, second rater second.}

\item{validate}{Logical; if \code{TRUE}, run Monte Carlo validation after model fitting.}

\item{R}{Number of Monte Carlo repetitions when \code{validate = TRUE}.}

\item{N}{Optional override for the sample size used in validation.}

\item{delta}{Optional override for the pre-post effect used in validation.}

\item{bias_r2}{Optional override for the additive bias of the second rater
used in validation.}

\item{n_draws}{Number of posterior draws used in model fitting.}

\item{seed}{Optional integer seed for reproducibility.}
}
\value{
A named list containing:
\describe{
\item{fit}{A \code{babebi_fit} object returned by \code{fit_model()}.}
\item{validation}{A \code{babebi_validation} object returned by
\code{montecarlo_from_data()} when \code{validate = TRUE}, otherwise \code{NULL}.}
\item{call}{The original function call.}
}
}
\description{
Fits the babebi model to observed data and, optionally, performs Monte Carlo
validation calibrated on the observed design.
}
\examples{
dat <- data.frame(
  id = rep(1:4, each = 4),
  time = rep(c("pre", "pre", "post", "post"), times = 4),
  rater = rep(c("r1", "r2"), times = 8),
  y = c(
    3.0, 3.2, 3.8, 4.0,
    2.9, 3.1, 3.5, 3.7,
    3.4, 3.6, 4.0, 4.1,
    3.1, 3.3, 3.9, 4.0
  )
)

res <- babebi(
  data = dat,
  id = id,
  time = time,
  rater = rater,
  outcome = y,
  time_order = c("pre", "post"),
  rater_order = c("r1", "r2"),
  validate = TRUE,
  R = 100,
  n_draws = 500,
  seed = 123
)

print(res$fit)
summary(res$fit)
plot(res$fit)
print(res$validation)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepare_data.R
\name{prepare_data}
\alias{prepare_data}
\title{Standardise user data for babebi}
\usage{
prepare_data(
  data,
  id,
  time,
  rater,
  outcome,
  time_order = NULL,
  rater_order = NULL
)
}
\arguments{
\item{data}{A data frame in long format.}

\item{id}{Unquoted column name identifying subjects or a character string.}

\item{time}{Unquoted column name identifying time points or a character string.}

\item{rater}{Unquoted column name identifying raters or a character string.}

\item{outcome}{Unquoted column name containing the observed score or a character string.}

\item{time_order}{Optional character vector of length 2 specifying the
order of the time factor (e.g., \code{c("pre", "post")}).}

\item{rater_order}{Optional character vector of length 2 specifying the
order of the rater factor (reference rater first).}
}
\value{
A data frame with four standardised columns: \code{id}, \code{time},
\code{rater}, and \code{y}.
}
\description{
Internal helper used to convert user-supplied data into the format
required by the estimation and Monte Carlo engine.
}
\details{
The function assumes a fully observed 2 (time) × 2 (rater) repeated-measures
design. Each subject must contribute exactly one observation for each
combination of time point and rater.

If \code{time_order} or \code{rater_order} are not provided, the ordering of factor
levels follows the order of appearance in the data. Users are encouraged
to specify these arguments explicitly when the ordering carries substantive
meaning (e.g., pre vs post).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{print.babebi_validation}
\alias{print.babebi_validation}
\title{Print a babebi validation object}
\usage{
\method{print}{babebi_validation}(x, digits = 3, ...)
}
\arguments{
\item{x}{An object of class \code{babebi_validation}.}

\item{digits}{Number of digits to print.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Print a babebi validation object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_design.R
\name{extract_design}
\alias{extract_design}
\title{Extract design parameters for Monte Carlo calibration}
\usage{
extract_design(data)
}
\arguments{
\item{data}{A data frame returned by \code{prepare_data}, with columns
\code{id}, \code{time}, \code{rater}, and \code{y}.}
}
\value{
A named list containing:
\describe{
\item{N}{Number of unique subjects.}
\item{delta}{Estimated mean pre-post change, averaged across raters.}
\item{bias_r2}{Estimated mean additive discrepancy of the second rater
relative to the reference rater, averaged across subjects and time points.}
\item{time_levels}{The two ordered time levels used in the data.}
\item{rater_levels}{The two ordered rater levels used in the data.}
}
}
\description{
Internal helper used to estimate sample size, mean pre-post change,
and second-rater additive discrepancy from standardised user-supplied
data for Monte Carlo calibration.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_cell.R
\name{run_cell}
\alias{run_cell}
\title{Run one Monte Carlo validation cell}
\usage{
run_cell(N, delta, bias_r2, R)
}
\arguments{
\item{N}{Integer sample size.}

\item{delta}{Numeric pre-post effect for the reference rater.}

\item{bias_r2}{Numeric additive discrepancy for the second rater.}

\item{R}{Integer number of Monte Carlo repetitions.}
}
\value{
A one-row data frame with columns \code{N}, \code{delta}, \code{bias_r2}, \code{bias},
\code{rmse}, \code{coverage}, \code{ppos_mean}, and \code{bf_mean}.
}
\description{
Internal helper used to evaluate the analytical babebi model under a single
simulation condition defined by sample size, pre-post effect, and rater bias.
}
\details{
The current implementation simulates pre-post difference scores directly for
the reference rater and the second rater, then applies the same analytical
linear-model logic used in \code{fit_model()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{summary.babebi_fit}
\alias{summary.babebi_fit}
\title{Summarise a babebi fit object}
\usage{
\method{summary}{babebi_fit}(object, digits = 3, ...)
}
\arguments{
\item{object}{An object of class \code{babebi_fit}.}

\item{digits}{Number of digits to print.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{summary.babebi_fit}.
}
\description{
Summarise a babebi fit object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resolve_colname.R
\name{resolve_colname}
\alias{resolve_colname}
\title{Resolve a column name supplied as symbol or character string}
\usage{
resolve_colname(expr, env = parent.frame())
}
\arguments{
\item{expr}{A captured expression representing a column name.}

\item{env}{The environment in which the expression should be resolved.}
}
\value{
A character string of length 1.
}
\description{
Internal helper for functions that accept either unquoted column names
or character strings.
}
\keyword{internal}
